@@ -1290,29 +1290,32 @@ We will always add all architectures your host natively supports.
 """)
 
     def x_or_create(self, cls, defaults=None, **kwargs):
-        if self.args["update"].value():
-            obj, created = cls.objects.update_or_create(**kwargs, defaults=defaults)
-        else:
-            obj, created = cls.objects.get_or_create(**kwargs, defaults=defaults)
+        _defaults = {} if defaults is None else defaults
 
-        class_info = self.result.setdefault(cls.__name__, {})
-        info = class_info.setdefault(str(obj), {})
+        info = {
+            "updated": False,
+            "created": False,
+            "diff": {},
+            "desc": "",
+        }
 
-        if created:
-            info["status"] = "created"
-        elif self.args["update"].value():
-            info["status"] = "updated"
-        else:
-            info["status"] = "exists"
-
-        # Add diff convenience; be sure we don't care if it fails, as code is somewhat whacky.
+        obj = None
         try:
-            info["diff"] = cls.mbd_diff(obj, **kwargs, **(defaults if defaults is not None else {}))
-        except BaseException as e:
-            mini_buildd.log_exception(LOG, f"Diff convenience feature failed for {cls}", e)
-            info["diff"] = None
+            LOG.debug(f"Found {cls.__name__} ({kwargs})")
+            obj = cls.objects.get(**kwargs)
+            info["diff"] = cls.mbd_diff(obj, **kwargs, **_defaults)
+            if self.args["update"].value():
+                info["updated"] = cls.objects.filter(**kwargs).update(**_defaults) > 0
+        except cls.DoesNotExist:
+            LOG.debug(f"NOT found {cls.__name__} ({kwargs})")
+            obj = cls(**kwargs, **_defaults)
+            obj.save()
+            info["created"] = True
 
-        return obj, created
+        info["desc"] = str(obj)
+
+        self.result.setdefault(cls.__name__, {})[obj.pk] = info
+        return obj, info["created"]
 
     @classmethod
     def ilocal_archive_urls(cls):
