@@ -124,7 +124,7 @@ def mbd_join(path, *paths):
 @register.filter
 def mbd_jsonpp(obj):
     """Get pretty print from json-serializable object."""
-    return json.dumps(obj, indent=2)
+    return mini_buildd.misc.json_pretty(obj)
 
 
 @register.simple_tag
@@ -146,6 +146,15 @@ def mbd_codename(diststr):
         return ""
 
 
+@register.filter
+def mbd_repository(diststr):
+    """Get repository from diststr. On any error, just return empty string."""
+    try:
+        return mini_buildd.dist.Dist(diststr).repository
+    except Exception:
+        return ""
+
+
 @register.simple_tag
 def mbd_model_stats(model):
     ret = {}
@@ -191,7 +200,7 @@ def mbd_api(context, call, name=None, hide_options=False, verbose=False, **kwarg
     args = {"mandatory": [], "optional": [], "hidden": []}
     for arg in api_call.args.values():
         args["mandatory" if arg.needs_value() else "hidden" if hide_options else "optional"].append(arg)
-    args["optional" if args["optional"] else "hidden"].insert(0, mini_buildd.api.Output())
+    args["optional" if args["optional"] else "hidden"].append(mini_buildd.api.Output())
 
     return {
         "request": context.get("request"),
@@ -215,6 +224,17 @@ def mbd_sbuild_status(bres):
 
 
 class PlainFilter(dict):
+    """
+    Enrich plain text with HTML hints (TOC)
+
+    .. note:: http://tools.ietf.org/html/rfc5147: This, in theory, could be some way to create a TOC for text/plain (buildlogs and such). However
+
+      * RFC only describes line/char based links, not text matching based (so scanning the text file would be needed still)
+      * As it seems, main browsers simply don't support it
+        * https://bugzilla.mozilla.org/show_bug.cgi?id=660583
+        * https://bugs.chromium.org/p/chromium/issues/detail?id=77024
+    """
+
     #: Allow (non 'text/') mime types
     ALLOWED_MIME_TYPES = ["application/json"]
     #: Allow extensions that have no official mime type
@@ -273,7 +293,7 @@ class PlainFilter(dict):
     @classmethod
     def filter_json(cls, file_path):
         with mini_buildd.fopen(file_path) as f:
-            return json.dumps(json.load(f), indent=2)
+            return mini_buildd.misc.json_pretty(json.load(f))
 
     @classmethod
     def filter_default(cls, file_path):
