@@ -24,10 +24,13 @@ class Chroot(mini_buildd.models.base.StatusModel):
     PERSONALITIES = {'i386': 'linux32'}
 
     MBD_HELP_EXTRA_OPTIONS = """
-Debootstrap-Command: <ALT_COMMAND>: Alternate command to run instead of standard debootstrap.
+Debootstrap-Command: <ALT_COMMAND>: Alternate command to run instead of standard debootstrap
 
-For example, ``Debootstrap-Command: /usr/sbin/qemu-debootstrap`` may be used to produce ``armel``
-chroots (with ``qemu-user-static`` installed).
+  For example, ``Debootstrap-Command: /usr/sbin/mini-buildd-debootstrap-uname-2.6`` may be used as workaround for urold (``<= lenny``) distributions.
+
+Debootstrap-Extra-Options: <EXTRA_OPTIONS>: Additional debootrap options
+
+  For example, ``--include=foo,bar`` may be used to have these packages pre-installed in build chroots.
 """
 
     source = django.db.models.ForeignKey(mini_buildd.models.source.Source,
@@ -65,10 +68,6 @@ chroots (with ``qemu-user-static`` installed).
                 fields.append("architecture")
             return fields
 
-        @classmethod
-        def mbd_host_architecture(cls):
-            return mini_buildd.models.source.Architecture.mbd_host_architecture()
-
     def mbd_key(self):
         return f"{self.source.codename}:{self.architecture.name}"
 
@@ -122,6 +121,8 @@ chroots (with ``qemu-user-static`` installed).
             LOG.warning(f"{self}: Can't get archive URL from source (source removed?): {e}")
             debootstrap_url = "no_archive_url_found_maybe_source_is_removed"
 
+        debootstrap_extra_options = shlex.split(self.mbd_get_extra_option("Debootstrap-Extra-Options", ""))
+
         return [
             (["/bin/mkdir", "--verbose", self.mbd_get_tmp_dir()],
              ["/bin/rm", "--recursive", "--one-file-system", "--force", self.mbd_get_tmp_dir()])] + self.mbd_get_backend().mbd_get_pre_sequence() + [
@@ -129,6 +130,7 @@ chroots (with ``qemu-user-static`` installed).
                    "--variant", "buildd",
                    "--keyring", self.mbd_get_keyring_file(),
                    "--arch", self.architecture.name,
+                   *debootstrap_extra_options,
                    self.source.codename,
                    self.mbd_get_tmp_dir(),
                    debootstrap_url],
