@@ -5,6 +5,7 @@ import logging
 import contextlib
 import functools
 import copy
+from contextlib import closing
 
 import django.db.models
 import django.utils.timezone
@@ -49,34 +50,48 @@ Use the 'directory' notation with exactly one trailing slash (like 'http://examp
         if self.url[-1] != "/" or self.url[-2] == "/":  # pylint: disable=unsubscriptable-object
             raise django.core.exceptions.ValidationError({"url": f"Invalid archive url '{self.url}': Must have exactly one trailing slash (like 'http://example.org/path/')."})
 
-    def mbd_get_matching_release(self, source, gnupg):
-        def download(file_, url):
-            with urllib.request.urlopen(url, timeout=self.URLOPEN_TIMEOUT) as response:
+    class ReleaseFile():
+        @classmethod
+        def _download(cls, file_, url):
+            with urllib.request.urlopen(url, timeout=Archive.URLOPEN_TIMEOUT) as response:
                 file_.write(response.read())
                 file_.flush()
                 file_.seek(0)
 
-        url_base = f"{self.url}/dists/{source.codename}/"
-        with tempfile.NamedTemporaryFile() as release_file, tempfile.NamedTemporaryFile() as signature_file:
-            signature_url = None
+        def __init__(self, url_base):
+            self.url_base = url_base
+
+            # pylint: disable=consider-using-with  # See close(), use this class with contextlib.closing()
+            self.release_file = tempfile.NamedTemporaryFile()
+            self.signature_file = tempfile.NamedTemporaryFile()
+            self.release_url, self.signature_url = None, None
+
             try:
-                release_url = url_base + "InRelease"
-                download(release_file, release_url)
+                self.release_url = self.url_base + "InRelease"
+                self._download(self.release_file, self.release_url)
             except urllib.error.HTTPError:
                 try:
-                    release_url = url_base + "Release"
-                    download(release_file, release_url)
-                    signature_url = release_url + ".gpg"
-                    download(signature_file, signature_url)
+                    self.release_url = self.url_base + "Release"
+                    self._download(self.release_file, self.release_url)
+                    self.signature_url = self.release_url + ".gpg"
+                    self._download(self.signature_file, self.signature_url)
                 except urllib.error.HTTPError as e:
-                    raise mini_buildd.HTTPUnavailable(f"{url_base}: Neither 'InRelease' nor 'Release[.gpg]' found") from e
+                    raise mini_buildd.HTTPNotFound(f"{self.url_base}: Neither 'InRelease' nor 'Release[.gpg]' found") from e
 
-            LOG.debug(f"Checking: {release_url} for {source}")
-            release = debian.deb822.Release(release_file)
+            self.release = debian.deb822.Release(self.release_file)
+
+        def close(self):
+            for t in [self.release_file, self.signature_file]:
+                if t is not None:
+                    t.close()
+
+        def match(self, source):
+            LOG.debug(f"match(): {self.release_url} matches {source}?")
 
             # Check release file fields
-            if not source.mbd_is_matching_release(release):
-                raise Exception(f"{release_url} does not match {source}')")
+            for key, value in list(source.mbd_release_file_values().items()):
+                if value != self.release.get(key):
+                    raise mini_buildd.HTTPNotFound(f"'{self.release_url}' does not match (has '{key}: {self.release.get(key)}', we need '{value}')")
 
             # Pre-Check 'Valid-Until'
             #
@@ -89,26 +104,20 @@ Use the 'directory' notation with exactly one trailing slash (like 'http://examp
             # This pre-check avoids such a situation, or at least it
             # can be fixed by re-checking the source.
             try:
-                valid_until = mini_buildd.misc.Datetime.parse(release["Valid-Until"])
+                valid_until = mini_buildd.misc.Datetime.parse(self.release["Valid-Until"])
                 if valid_until < mini_buildd.misc.Datetime.now():
                     if source.mbd_get_extra_option("X-Check-Valid-Until", "yes").lower() in ("no", "false", "0"):
-                        LOG.warning(f"{release_url} expired, but source marked to ignore valid-until (Valid-Until='{valid_until}').")
+                        LOG.warning(f"{self.release_url} expired, but source marked to ignore valid-until (Valid-Until='{valid_until}').")
                     else:
-                        LOG.warning(f"{release_url} expired, maybe the archive has problems? (Valid-Until='{valid_until}').")
-                        raise Exception(f"{release_url} expired")
+                        LOG.warning(f"{self.release_url} expired, maybe the archive has problems? (Valid-Until='{valid_until}').")
+                        raise mini_buildd.HTTPUnavailable(f"{self.release_url} expired")
             except KeyError:
                 pass  # We can assume Release file has no "Valid-Until", and be quiet
             except BaseException as e:
-                mini_buildd.log_exception(LOG, f"Ignoring error checking 'Valid-Until' on {release_url}", e)
+                mini_buildd.log_exception(LOG, f"Ignoring error checking 'Valid-Until' on {self.release_url}", e)
 
-            # Check signature. This also logs *all* keys the release file was signed with (possibly hinting to clean up setup).
-            gnupg.verify_release(release_file, signature_file if signature_url is not None else None, accept_expired=True, log_key=source.codename)
-
-            # User success info.
-            LOG.info(f"'{source}' found via '{release_url}'")
-
-            # Ok, this is for this source
-            return release
+        def verify(self, gnupg):
+            return gnupg.gpgme_verify_release(self.release_file, self.signature_file if self.signature_url is not None else None, accept_expired=True)
 
     def mbd_ping(self):
         """Ping and update the ping value."""
@@ -132,7 +141,7 @@ Use the 'directory' notation with exactly one trailing slash (like 'http://examp
         except Exception as e:
             self.ping = -1.0
             self.save()
-            raise Exception(f"{self}: Does not ping: {e}") from e
+            raise mini_buildd.HTTPUnavailable(f"'{self}' does not ping: {e}") from e
 
     def mbd_get_reverse_dependencies(self):
         """Return all sources (and their deps) that use us."""
@@ -148,17 +157,6 @@ class Architecture(mini_buildd.models.base.Model):
     def __str__(self):
         return f"{self.name}"
 
-    @classmethod
-    def mbd_host_architecture(cls):
-        return mini_buildd.call.Call(["dpkg", "--print-architecture"]).check().stdout.strip()
-
-    @classmethod
-    def mbd_supported_architectures(cls, arch=None):
-        """Get all supported architectures (some archs also natively support other archs)."""
-        arch = arch or cls.mbd_host_architecture()
-        arch_map = {"amd64": ["i386"]}
-        return [arch] + arch_map.get(arch, [])
-
 
 @functools.total_ordering
 class Component(mini_buildd.models.base.Model):
@@ -303,11 +301,11 @@ Some sources need some more care via ``Extra options``, see below.
         def mbd_filter_active_base_sources(cls):
             """Filter active base sources; needed in chroot and distribution wizards."""
             return Source.objects.filter(status__gte=Source.STATUS_ACTIVE,
-                                         origin__in=["Debian", "Ubuntu"],
+                                         origin__in=mini_buildd.dist.SETUP["origin"].keys(),
                                          codename__regex=r"^[a-z]+$").exclude(codename="experimental")
 
     def __str__(self):
-        return f"{self.origin} {self.codeversion} ({self.codename})"
+        return f"{self.origin} '{self.codename}' ({self.codeversion})"
 
     def mbd_release_file_values(self):
         """Compute a dict of values a matching release file must have."""
@@ -320,20 +318,12 @@ Some sources need some more care via ``Extra options``, see below.
 
         return values
 
-    def mbd_is_matching_release(self, release):
-        """Check that this release file matches us."""
-        for key, value in list(self.mbd_release_file_values().items()):
-            # Check identity: origin, codename
-            if value != release[key]:
-                return False
-        return True
-
     def mbd_get_archive(self):
         """Get fastest archive."""
         oa_list = self.archives.all().filter(ping__gte=0.0).order_by("ping")
         if oa_list:
             return oa_list[0]
-        raise mini_buildd.HTTPBadRequest(f"{self}: No archive found. Please add appropriate archive and/or check network setup.")
+        raise mini_buildd.HTTPBadRequest(f"{self}: No archive found (network down? ``Archive`` missing? ``Source`` with incorrect keys?)")
 
     def mbd_get_apt_line(self, limit_components=None):
         components = [c for c in self.components.all() if limit_components is None or c in limit_components]
@@ -370,45 +360,54 @@ Some sources need some more care via ``Extra options``, see below.
     def mbd_check(self):
         """Rescan all archives, and check that there is at least one working."""
         self.archives.set([])
+
+        # Gather failed information to debug if no archive was found
+        failed = {}
+
         with contextlib.closing(mini_buildd.gnupg.TmpGnuPG(tmpdir_options={"prefix": "gnupg-sourcecheck-"})) as gpg:
             for k in self.apt_keys.all():
                 gpg.add_pub_key(k.key)
 
             for archive in Archive.objects.all():
                 try:
-                    # Check and update the ping value
                     archive.mbd_ping()
 
-                    # Get release if this archive serves us, else exception
-                    release = archive.mbd_get_matching_release(self, gpg)
+                    with closing(Archive.ReleaseFile(f"{archive.url}/dists/{self.codename}/")) as release_file:
+                        release_file.match(self)
+                        release_file.verify(gpg)
 
-                    # Implicitly save ping value for this archive
-                    self.archives.add(archive)
-                    self.description = release["Description"]
+                        # Implicitly save ping value for this archive
+                        self.archives.add(archive)
+                        self.description = release_file.release.get("Description", "")
 
-                    # Set codeversion
-                    self.codeversion = ""
-                    if self.codeversion_override:
-                        self.codeversion = self.codeversion_override
-                    else:
-                        self.codeversion = mini_buildd.dist.guess_codeversion(release)
-                        self.codeversion_override = self.codeversion
+                        # Set codeversion
+                        self.codeversion = ""
+                        if self.codeversion_override:
+                            self.codeversion = self.codeversion_override
+                        else:
+                            self.codeversion = mini_buildd.dist.guess_codeversion(release_file.release.get("Origin"), release_file.release.get("Codename"), release_file.release.get("Version"))
+                            self.codeversion_override = self.codeversion
 
-                    # Set architectures and components (may be auto-added)
-                    if release.get("Architectures"):
-                        for a in release["Architectures"].split(" "):
-                            new_arch, _created = Architecture.objects.get_or_create(name=a)
-                            self.architectures.add(new_arch)
-                    if release.get("Components"):
-                        for c in release["Components"].split(" "):
-                            new_component, _created = Component.objects.get_or_create(name=c.replace(self.mbd_get_extra_option("X-Remove-From-Component", ""), ""))
-                            self.components.add(new_component)
-                except BaseException as e:
-                    mini_buildd.log_exception(LOG, f"'{self}' not hosted by '{archive}'", e, level=logging.INFO)
+                        # Set architectures and components (may be auto-added)
+                        if release_file.release.get("Architectures"):
+                            for a in release_file.release["Architectures"].split(" "):
+                                new_arch, _created = Architecture.objects.get_or_create(name=a)
+                                self.architectures.add(new_arch)
+                        if release_file.release.get("Components"):
+                            for c in release_file.release["Components"].split(" "):
+                                new_component, _created = Component.objects.get_or_create(name=c.replace(self.mbd_get_extra_option("X-Remove-From-Component", ""), ""))
+                                self.components.add(new_component)
+                except mini_buildd.HTTPError as e:
+                    mini_buildd.log_exception(LOG, f"{self}: Skipping {archive}", e, level=logging.DEBUG)
+                    failed[archive] = e
 
         # Check that at least one archive can be found
-        fastest_archive = self.mbd_get_archive()
-        LOG.info(f"{self}: Fastest archive: {fastest_archive}")
+        try:
+            fastest_archive = self.mbd_get_archive()
+            LOG.debug(f"{self}: Fastest archive: {fastest_archive}")
+        except mini_buildd.HTTPError as e:
+            errs = '\n'.join([f'{archive}: {error}' for archive, error in failed.items()])
+            raise mini_buildd.HTTPNotFound(f"{e.detail}:\n{errs}") from e
 
     def mbd_get_dependencies(self):
         return self.apt_keys.all()
@@ -424,17 +423,12 @@ Some sources need some more care via ``Extra options``, see below.
 
     def __eq__(self, other):
         if isinstance(other, self.__class__):
-            return self.origin == other.origin and self.codename == other.codename
+            return mini_buildd.dist.Codename(self.codename, origin=self.origin) == mini_buildd.dist.Codename(other.codename, origin=other.origin)
         return NotImplemented
 
     def __lt__(self, other):
         if isinstance(other, self.__class__):
-            if self.origin == other.origin:
-                try:
-                    return int(self.codeversion) < int(other.codeversion)
-                except BaseException:
-                    return self.codeversion < other.codeversion
-            return self.origin < other.origin
+            return mini_buildd.dist.Codename(self.codename, origin=self.origin) < mini_buildd.dist.Codename(other.codename, origin=other.origin)
         return NotImplemented
 
     def clean(self, *args, **kwargs):
