@@ -423,7 +423,7 @@ class SBuild():
         """
         Run sbuild && update buildresult from build log.
 
-        .. important:: This will iterate all lines of the build log,
+        .. note:: This will iterate all lines of the build log,
                   and parse out the selection of sbuild's summary status
                   we need.  In case the build log above does write the
                   same output like 'Status: xyz', sbuild's correct status
