@@ -112,8 +112,7 @@ class DefaultView(TemplateView):
 class HomeView(DefaultView):
     def setup(self, request, *args, **kwargs):
         super().setup(request, *args, **kwargs)
-        self.mbd_events_view_max = mini_buildd.get_daemon().model.show_last_builds
-        self.mbd_events, self.mbd_events_total, self.mbd_events_max, self.mbd_events_ongoing = mini_buildd.get_daemon().events.mbd_slice(maxlen=self.mbd_events_view_max)
+        self.mbd_events_ongoing = mini_buildd.get_daemon().events.ongoing.values()
 
 
 class AccountRegisterView(django.views.generic.edit.FormView):
